<?php

function dci_admin_dashboard_page() {

  $left = array(
    'progress' => _dci_admin_dashboard_progress_block(),
    'pending_objects' => _dci_admin_dashboard_pending_objects_block(),
  );

  $right = array(
    'user_activity' => _dci_admin_dashboard_user_activity_block(),
    'groups' => _dci_admin_dashboard_groups_block(),
    'content_review' => _dci_admin_dashboard_content_review_block(),
  );

  $page = array(
    '#theme' => 'dci_admin_dashboard',
    'left' => $left,
    'right' => $right,
  );

  drupal_set_title(t('Distance Cataloging Interface'));

  drupal_alter('dci_admin_dashboard', $page);

  return $page;
}

function _dci_admin_dashboard_progress_block() {

  $total = $month = $week = 0;

  $dci_types = dci_node_types();

  if (!empty($dci_types)) {

    $query = db_select('node', 'n');
    $query->addTag('dci_content');
    $query->join('field_data_dci_nodes', 'fn', 'fn.dci_nodes_target_id = n.nid');
    $query->join('field_data_dci_users', 'fu', 'fu.entity_id = fn.entity_id');
    $query->join('dci_group', 'g', 'g.gid = fn.entity_id');
    $query->leftJoin('dci_feedback', 'f', 'fn.dci_nodes_target_id = f.nid AND fu.dci_users_target_id = f.uid');
    $query->condition('n.type', $dci_types, 'IN');
    $query->addExpression('COUNT(*)', 'total');
    $query->addExpression('COUNT(f.status)', 'reviewed');

    $result = $query->execute()->fetchObject();
    $total = $result->total > 0 ? min(100, intval($result->reviewed * 100 / $result->total)) : ($result->reviewed > 0 ? 100 : 0);

    $query = db_select('node', 'n');
    $query->addTag('dci_content');
    $query->join('field_data_dci_nodes', 'fn', 'fn.dci_nodes_target_id = n.nid');
    $query->join('field_data_dci_users', 'fu', 'fu.entity_id = fn.entity_id');
    $query->leftJoin('dci_group', 'g', 'g.gid = fn.entity_id AND g.due < UNIX_TIMESTAMP(DATE_ADD(DATE_SUB(CURDATE(), INTERVAL DAYOFMONTH(CURDATE()) - 1 DAY), INTERVAL 1 MONTH))');
    $query->leftJoin('dci_feedback', 'f', 'fn.dci_nodes_target_id = f.nid AND fu.dci_users_target_id = f.uid');
    $query->condition('n.type', $dci_types, 'IN');
    $query->addExpression('COUNT(g.gid)', 'total');
    $query->addExpression('COUNT(f.status)', 'reviewed');

    $result = $query->execute()->fetchObject();
    $month = $result->total > 0 ? min(100, intval($result->reviewed * 100 / $result->total)) : ($result->reviewed > 0 ? 100 : 0);

    $query = db_select('node', 'n');
    $query->addTag('dci_content');
    $query->join('field_data_dci_nodes', 'fn', 'fn.dci_nodes_target_id = n.nid');
    $query->join('field_data_dci_users', 'fu', 'fu.entity_id = fn.entity_id');
    $query->leftJoin('dci_group', 'g', 'g.gid = fn.entity_id AND g.due < UNIX_TIMESTAMP(DATE_ADD(DATE_SUB(CURDATE(), INTERVAL WEEKDAY(CURDATE()) DAY), INTERVAL 1 WEEK))');
    $query->leftJoin('dci_feedback', 'f', 'fn.dci_nodes_target_id = f.nid AND fu.dci_users_target_id = f.uid');
    $query->condition('n.type', $dci_types, 'IN');
    $query->addExpression('COUNT(g.gid)', 'total');
    $query->addExpression('COUNT(f.status)', 'reviewed');

    $result = $query->execute()->fetchObject();
    $week = $result->total > 0 ? min(100, intval($result->reviewed * 100 / $result->total)) : ($result->reviewed > 0 ? 100 : 0);
  }

  $block = array(
    '#theme' => 'dci_admin_dashboard_block__progress',
    'title' => t('Progress'),
    'content' => array(
      'week' => array(
        '#type' => 'item',
        '#title' => t('This Week'),
        '#markup' => $week . '%',
        '#value' => $week,
        '#color' => '#6cc6d6',
      ),
      'month' => array(
        '#type' => 'item',
        '#title' => t('This Month'),
        '#markup' => $month . '%',
        '#value' => $month,
        '#color' => '#79b099',
      ),
      'total' => array(
        '#type' => 'item',
        '#title' => t('Overall Progress'),
        '#markup' => $total . '%',
        '#value' => $total,
        '#color' => '#4980b8',
      ),
    ),
  );

  return $block;
}



function _dci_admin_dashboard_pending_objects_block() {
  
  $block = array(
    '#theme' => 'dci_admin_dashboard_block__pending_objects',
    'title' => t('Pending Objects'),
    'content' => array(
      'pending_objects' => drupal_get_form('_dci_pending_nodes'),
    ),
  );

  return $block;
}

function _dci_admin_dashboard_user_activity_block() {

  $interval = REQUEST_TIME - 900; // authenticated within last 15 minutes

  $authenticated_count = db_query("SELECT COUNT(DISTINCT s.uid) FROM {sessions} s WHERE s.timestamp >= :timestamp AND s.uid > 0", array(':timestamp' => $interval))->fetchField();
  $blocked_count = db_query("SELECT COUNT(DISTINCT u.uid) FROM {users} u WHERE u.uid > 0 AND u.status = 0")->fetchField();


  $query = db_select('users', 'u');
  $query->join('flood', 'f', "f.identifier LIKE CONCAT(u.uid, '%')");
  $query->addExpression('COUNT(DISTINCT u.uid)');
  $query->condition('f.expiration', REQUEST_TIME, '>');
  $query->condition('u.status', 1);

  $locked_out = $query->execute()->fetchField();

  $query = db_select('users', 'u')->condition('u.status', 1);
  $query->addExpression('COUNT(u.uid)');

  $sq1 = db_select('users_roles', 'ur')->fields('ur', array('uid'));
  $sq1->join('role_permission', 'rp', 'rp.rid = ur.rid');
  $sq1->condition('rp.permission', 'submit content feedback');

  $sq2 = db_select('field_data_dci_users', 'du')
      ->fields('du', array('dci_users_target_id'))
      ->condition('du.deleted', 0);

  $query->condition('u.uid', $sq1, 'IN');
  $query->condition('u.uid', $sq2, 'IN');

  $dci_users_with_items = $query->execute()->fetchField();

  $query = db_select('users', 'u')->condition('u.status', 1);
  $query->addExpression('COUNT(u.uid)');
  $query->condition('u.uid', $sq1, 'IN');

  $sq2 = db_select('dci_feedback', 'f')
      ->fields('f', array('uid'));

  $query->condition('u.uid', $sq2, 'IN');

  $dci_users_with_feedback = $query->execute()->fetchField();

  $active_pct = $dci_users_with_items ? ($dci_users_with_feedback / $dci_users_with_items) * 100 : 0;
  
  $block = array(
    '#theme' => 'dci_admin_dashboard_block__user_activity',
    'title' => t('User Activity'),
    'content' => array(
      'logged_in' => array(
        '#type' => 'item',
        '#title' => t('Logged in users'),
        '#markup' => $authenticated_count,
        '#value' => $authenticated_count,
      ),
      'unapproved' => array(
        '#type' => 'item',
        '#title' => t('Unapproved users'),
        '#markup' => $blocked_count,
        '#value' => $blocked_count,
      ),
      'locked_out' => array(
        '#type' => 'item',
        '#title' => t('Locked out'),
        '#markup' => $locked_out,
        '#value' => $locked_out,
      ),
      'active' => array(
        '#type' => 'item',
        '#title' => t('Active Users'),
        '#markup' => sprintf("%.0f%%", $active_pct),
        '#value' => $active_pct,
      ),
    ),
  );

  return $block;
}

function _dci_admin_dashboard_groups_block() {
  
  $block = array(
    '#theme' => 'dci_admin_dashboard_block__groups',
    'title' => t('Groups'),
    'content' => array(),
  );

  $destination = drupal_get_destination();

  $query = db_select('dci_group', 'g')
      ->fields('g', array('gid', 'title'))
      ->range(0, 3)
      ->orderBy('g.created', 'DESC');

  $groups = $query->execute();

  $options = array();
  $data = array();

  foreach ($groups as $group) {

    $data[] = $group;

    $options[$group->gid] = array(
      'title' => array(
        'data' => array(
          '#type' => 'link',
          '#title' => $group->title,
          '#href' => 'admin/dci/groups/' . $group->gid,
        ),
      ),
    );
    
    // Build a list of all the accessible operations for the current node.
    $operations = array();

    $operations['assign'] = array(
      'title' => t('assign'),
      'href' => 'admin/dci/groups/' . $group->gid . '/assign',
      'query' => $destination,
    );
   
    $options[$group->gid]['operations'] = array();
    $options[$group->gid]['operations'] = array(
      'data' => array(
        '#theme' => 'links',
        '#links' => $operations,
        '#attributes' => array('class' => array('links', 'inline')),
      ),
    );   
  }


  $block['content']['recently_created'] = array(
    '#theme' => 'table',
    '#caption' => t('Recently Created Groups'),
    '#header' => array(),
    '#rows' => $options,
    '#empty' => t('No content available.'),
    '#raw_data' => $data,
  );

  $subquery = db_select('field_data_dci_nodes', 'fn');
  $subquery->join('field_data_dci_users', 'fu', 'fu.entity_id = fn.entity_id');
  $subquery->addJoin('LEFT OUTER', 'dci_feedback', 'f', 'fn.dci_nodes_target_id = f.nid AND fu.dci_users_target_id = f.uid');
  $subquery->addField('fn', 'entity_id', 'gid');
  $subquery->addExpression('COUNT(*)', 'total_feedback');
  $subquery->addExpression('COUNT(f.nid)', 'submitted_feedback');
  $subquery->addExpression('COUNT(f.status)', 'reviewed_feedback');
  $subquery->addExpression('COUNT(SIGN(f.status)) - SUM(SIGN(f.status))', 'rejected_feedback');
  $subquery->addExpression('SUM(SIGN(f.status))', 'approved_feedback');
  $subquery->groupBy('fn.entity_id');


  $query = db_select('dci_group', 'g')
      ->fields('g', array('gid', 'title'))
      ->range(0, 3)
      ->orderBy('g.changed', 'DESC');

  $query->join($subquery, 'gf', 'gf.gid = g.gid');
  $query->addField('gf', 'total_feedback');
  $query->addField('gf', 'reviewed_feedback');

  $groups = $query->execute();

  $options = array();
  $data = array();

  foreach ($groups as $group) {

    $progress = 100 * ($group->total_feedback > 0 ? $group->reviewed_feedback / $group->total_feedback : 0);

    $group->progress = $progress;

    $data[] = $group;
    
    $options[$group->gid] = array(
      'title' => array(
        'data' => array(
          '#type' => 'link',
          '#title' => $group->title,
          '#href' => 'admin/dci/groups/' . $group->gid,
        ),        
      ),
      'progress' => sprintf("%.0f%%", $progress),
    );
    
    // Build a list of all the accessible operations for the current node.
    $operations = array();

    $operations['assign'] = array(
      'title' => t('assign'),
      'href' => 'admin/dci/groups/' . $group->gid . '/assign',
      'query' => $destination,
    );
   
    $options[$group->gid]['operations'] = array();
    $options[$group->gid]['operations'] = array(
      'data' => array(
        '#theme' => 'links',
        '#links' => $operations,
        '#attributes' => array('class' => array('links', 'inline')),
      ),
    );   
  }


  $block['content']['recently_assigned'] = array(
    '#theme' => 'table',
    '#caption' => t('Recently Assigned Groups'),
    '#header' => array(),
    '#rows' => $options,
    '#empty' => t('No content available.'),
    '#raw_data' => $data,
  );



  $query = db_select('dci_group', 'g')
      ->fields('g', array('gid', 'title'))
      ->range(0, 3);

  $subquery->addExpression('MAX(f.reviewed)', 'reviewed');      

  $query->join($subquery, 'gf', 'gf.gid = g.gid AND gf.total_feedback = gf.reviewed_feedback');

  $query->orderBy('gf.reviewed', 'DESC');

  $groups = $query->execute();

  $options = array();
  $data = array();

  foreach ($groups as $group) {

    $data[] = $group;
    
    $options[$group->gid] = array(
      'title' => array(
        'data' => array(
          '#type' => 'link',
          '#title' => $group->title,
          '#href' => 'admin/dci/groups/' . $group->gid,
        ),
      ),
    );
    
    // Build a list of all the accessible operations for the current node.
    $operations = array();

    $operations['view'] = array(
      'title' => t('view'),
      'href' => 'admin/dci/groups/' . $group->gid,
      'query' => $destination,
    );
   
    $options[$group->gid]['operations'] = array();
    $options[$group->gid]['operations'] = array(
      'data' => array(
        '#theme' => 'links',
        '#links' => $operations,
        '#attributes' => array('class' => array('links', 'inline')),
      ),
    );   
  }

  $block['content']['recently_completed'] = array(
    '#theme' => 'table',
    '#caption' => t('Recently Completed Groups'),
    '#header' => array(),
    '#rows' => $options,
    '#empty' => t('No content available.'),
    '#raw_data' => $data,
  );

  $block['content']['links'] = array(
    '#theme' => 'links',
    '#links' => array(
      'view-all-groups' => array(
        'title' => t('View All Groups'),
        'href' => 'admin/dci/groups',
      ),
      'create-group' => array(
        'title' => t('Create Group'),
        'href' => 'admin/dci/groups/add',
        'query' => $destination,
      ),
    ),
    '#attributes' => array('class' => array('links', 'inline')),
  );


  return $block;
}

function _dci_admin_dashboard_content_review_block() {

  $block = array(
    '#theme' => 'dci_admin_dashboard_block__content_review',
    'title' => t('Content that needs review'),
    'content' => array(),
  );

  $header = array(
    'title' => t('Title'),
    'assigned_to' => t('Assigned to'),
    'operations' => t('Control'),
  );

  $destination = drupal_get_destination();

  $subquery = db_select('dci_feedback', 'f');
  $subquery->isNull('f.status');
  $subquery->addField('f', 'nid');
  $subquery->addExpression('MAX(uid)', 'uid');
  $subquery->addExpression('MAX(changed)', 'changed');
  $subquery->groupBy('f.nid');  

  $query = db_select('node', 'n')      
    ->fields('n', array('title', 'nid'))    
    ->range(0, 5)
    ->orderBy('f.changed', 'DESC');

  $query->addTag('dci_content');

  $query->join($subquery, 'f', 'f.nid = n.nid');
  $query->join('users', 'u', 'u.uid = f.uid');

  $query->addField('f', 'uid');
  $query->addField('u', 'name', 'username');

  $groups = $query->execute();

  $options = array();

  $data = array();

  foreach ($groups as $node) {

    $data[] = $node;
    
    $options[$node->nid] = array(
      'title' => array(
        'data' => array(
          '#type' => 'link',
          '#title' => $node->title,
          '#href' => 'node/' . $node->nid . '/dci',
        ),        
      ),
      'assigned_to' => array(
        'data' => array(
          '#type' => 'link',
          '#title' => $node->username,
          '#href' => 'user/' . $node->uid,
        ),        
      ),
    );
    
    // Build a list of all the accessible operations for the current node.
    $operations = array();

    $operations['view'] = array(
      'title' => t('view'),
      'href' => 'node/' . $node->nid . '/dci',
      'query' => $destination,
    );
   
    $options[$node->nid]['operations'] = array();
    $options[$node->nid]['operations'] = array(
      'data' => array(
        '#theme' => 'links',
        '#links' => $operations,
        '#attributes' => array('class' => array('links', 'inline')),
      ),
    );   
  }

  $block['content']['review'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $options,
    '#empty' => t('No content available.'),
    '#raw_data' => $data,
  );

  $block['content']['links'] = array(
    '#theme' => 'links',
    '#links' => array(
      'review' => array(
        'title' => t('View All Pending Review'),
        'href' => 'admin/dci/review',
      ),
    ),
    '#attributes' => array('class' => array('links', 'inline')),
  );

  return $block;
}

/**
 * Form builder: Builds the node administration overview.
 *
 * @see node_admin_nodes_submit()
 * @see node_admin_nodes_validate()
 * @see node_filter_form()
 * @see node_filter_form_submit()
 * @see node_multiple_delete_confirm()
 * @see node_multiple_delete_confirm_submit()
 *
 * @ingroup forms
 */
function _dci_pending_nodes() {

  $dci_types = dci_node_types();
  
  // Build the sortable table header.
  $header = array(
    'title' => array('data' => t('Title'), 'field' => 'n.title'),
    'type' => array('data' => t('Type'), 'field' => 'n.type'),
    'author' => t('Author'),
    'status' => array('data' => t('Status'), 'field' => 'n.status'),
    'changed' => array('data' => t('Updated'), 'field' => 'n.changed', 'sort' => 'desc')
  );
  
  $header['operations'] = array('data' => t('Operations'));  

  $query = db_select('node', 'n')->extend('PagerDefault')->extend('TableSort');
  $query->addTag('dci_content');

  $sq1 = db_select('field_data_dci_nodes', 'dn')->fields('dn')->where('dn.dci_nodes_target_id = n.nid');
  $sq2 = db_select('dci_feedback', 'df')->fields('df')->where('df.nid = n.nid');
  
  $query->fields('n',array('nid'))
        ->notExists($sq1)
        ->notExists($sq2);

  if (!empty($dci_types)) {
    $query->condition('n.type', $dci_types, 'IN');
  }
  else {
   $query->condition(1, 0); 
  }
  
  $nids = $query->limit(10)
    ->orderByHeader($header)
    ->orderBy('n.nid', 'DESC')
    ->execute()
    ->fetchCol();


  $nodes = node_load_multiple($nids);  

  // Prepare the list of nodes.
  $languages = language_list();
  $destination = drupal_get_destination();
  $options = array();
  foreach ($nodes as $node) {
    $langcode = entity_language('node', $node);
    $l_options = $langcode != LANGUAGE_NONE && isset($languages[$langcode]) ? array('language' => $languages[$langcode]) : array();
    $options[$node->nid] = array(
      'title' => array(
        'data' => array(
          '#type' => 'link',
          '#title' => $node->title,
          '#href' => 'node/' . $node->nid . '/dci',
          '#options' => $l_options,
        ),
      ),
      'type' => check_plain(node_type_get_name($node)),
      'author' => theme('username', array('account' => $node)),
      'status' => $node->status ? t('published') : t('not published'),
      'changed' => format_date($node->changed, 'short'),
    );
    
    // Build a list of all the accessible operations for the current node.
    $operations = array();

    $operations['add_to_group'] = array(
      'title' => t('add to group'),
      'href' => 'admin/dci/' . $node->nid . '/add-to-group',
      'query' => $destination,
    );
    
    $options[$node->nid]['operations'] = array();
    if (count($operations) > 1) {
      // Render an unordered list of operations links.
      $options[$node->nid]['operations'] = array(
        'data' => array(
          '#theme' => 'links__node_operations',
          '#links' => $operations,
          '#attributes' => array('class' => array('links', 'inline')),
        ),
      );
    }
    elseif (!empty($operations)) {
      // Render the first and only operation as a link.
      $link = reset($operations);
      $options[$node->nid]['operations'] = array(
        'data' => array(
          '#type' => 'link',
          '#title' => $link['title'],
          '#href' => $link['href'],
          '#options' => array('query' => $link['query']),
        ),
      );
    }
  }

  $form['nodes'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $options,
    '#empty' => t('No content available.'),
    '#raw_data' => $nodes,
  );

  $form['pager'] = array('#theme' => 'pager');
  return $form;
}


function dci_admin_groups_page() {

  $page = array(
    '#theme' => 'dci_admin_groups',
    'content' => array(
      'form' => drupal_get_form('dci_admin_groups_form'),
    ),
  );
  drupal_set_title(t('Distance Cataloging Interface'));
  drupal_alter('dci_admin_groups', $page);

  return $page;
}

function dci_admin_content_review_page() {
  drupal_set_title(t('Distance Cataloging Interface'));
  $result = array(
    '#theme' => 'dci_admin_content_review',
    'content' => drupal_get_form('dci_admin_content_review_form'),
  );
  return $result;
}

/**
 * Page callback: Form constructor for the content administration form.
 */
function dci_admin_groups_form($form, $form_state) {
  
  $form['filter'] = dci_admin_groups_filter_form();
  $form['#submit'][] = 'dci_admin_groups_filter_form_submit';
  $form['groups'] = dci_admin_groups();

  return $form;
}

function dci_group_filters() {
  // Regular filters
  $filters['title'] = array(
    'title' => t('Title'),
    'type' => 'textfield',
  );

  $filters['user'] = array(
    'title' => t('User'),
    'type' => 'textfield',
  );

  $filters['status'] = array(
    'title' => t('Moderation status'),
    'type' => 'select',
    'options' => array(
      '[any]' => t('any'),
      'not-assigned' => t('Not yet assigned'),
      'assigned' => t('Assigned'),
      'needs-review' => t('Needs review'),
      'approved' => t('Approved'),      
    ),
  );
 
  return $filters;
}

/**
 * Returns the node administration filters form array to node_admin_content().
 *
 * @see node_admin_nodes()
 * @see node_admin_nodes_submit()
 * @see node_admin_nodes_validate()
 * @see node_filter_form_submit()
 * @see node_multiple_delete_confirm()
 * @see node_multiple_delete_confirm_submit()
 *
 * @ingroup forms
 */
function dci_admin_groups_filter_form() {
  $session = isset($_SESSION['dci_groups_filter']) ? $_SESSION['dci_groups_filter'] : array();
  $filters = dci_group_filters();

  $i = 0;
  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Show only items where'),
    '#theme' => 'exposed_filters',
  );

  $form['filters']['status'] = array(
    '#type' => 'container',
    '#attributes' => array('class' => array('container-inline', 'clearfix')),
    '#prefix' => ($i && count($filters) > 0 ? '<div class="additional-filters">' . t('and where') . '</div>' : ''),
  );
  $form['filters']['status']['filters'] = array(
    '#type' => 'container',
    '#attributes' => array('class' => array('container-inline', 'filters')),
  );
  foreach ($filters as $key => $filter) {
    
    if ($filter['type'] == 'select') {
      $form['filters']['status']['filters'][$key] = array(
        '#type' => 'select',
        '#options' => $filter['options'],
        '#title' => $filter['title'],
        '#default_value' => isset($session[$key]) ? $session[$key] : '[any]',
      );
    }
    elseif ($filter['type'] == 'textfield') {
      $form['filters']['status']['filters'][$key] = array(
        '#type' => 'textfield',
        '#title' => $filter['title'],
        '#size' => 30,
        '#default_value' => isset($session[$key]) ? $session[$key] : NULL,
      );
    }
  }

  $form['filters']['status']['actions'] = array(
    '#type' => 'actions',
    '#attributes' => array('class' => array('container-inline')),
  );
  if (count($filters)) {
    $form['filters']['status']['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Filter'),
    );
  }
  if (count($session)) {    
    $form['filters']['status']['actions']['reset'] = array('#type' => 'submit', '#value' => t('Reset'));
  }

  drupal_add_js('misc/form.js');

  return $form;
}


/**
 * Form submission handler for node_filter_form().
 *
 * @see node_admin_content()
 * @see node_admin_nodes()
 * @see node_admin_nodes_submit()
 * @see node_admin_nodes_validate()
 * @see node_filter_form()
 * @see node_multiple_delete_confirm()
 * @see node_multiple_delete_confirm_submit()
 */
function dci_admin_groups_filter_form_submit($form, &$form_state) {  

  switch ($form_state['values']['op']) {
    case t('Filter'):
      $filters = dci_group_filters();
    
      // Apply every filter with a valid value.
      foreach ($filters as $filter => $info) {

        if (!empty($form_state['values'][$filter])) {
          switch ($info['type']) {
            
            case 'select':
              if ($form_state['values'][$filter] === '[any]') {
                unset($_SESSION['dci_groups_filter'][$filter]);
              }
              else {
                // Flatten the options array to accommodate hierarchical/nested options.
                $flat_options = form_options_flatten($info['options']);
                // Only accept valid selections offered on the dropdown, block bad input.
                if (isset($flat_options[$form_state['values'][$filter]])) {
                  $_SESSION['dci_groups_filter'][$filter] = $form_state['values'][$filter];
                }
              }
              break;

            case 'textfield':
              $_SESSION['dci_groups_filter'][$filter] = $form_state['values'][$filter];
              break;
          }
        }
        else {
          unset($_SESSION['dci_groups_filter'][$filter]);
        }
      }
      break;
    case t('Reset'):
      $_SESSION['dci_groups_filter'] = array();
      break;
  }
}

/**
 * Applies filters for node administration filters based on session.
 *
 * @param $query
 *   A SelectQuery to which the filters should be applied.
 */
function dci_groups_build_filter_query(SelectQueryInterface $query) {
  // Build query
  $filter_data = isset($_SESSION['dci_groups_filter']) ? $_SESSION['dci_groups_filter'] : array();
  foreach ($filter_data as $filter => $value) {
    
    switch ($filter) {
      case 'title':
        $query->condition('g.title', $value, 'LIKE');
        break;
      case 'user':
        $subquery = db_select('users', 'u')->condition('u.name', $value, 'LIKE');
        $subquery->join('field_data_dci_users', 'f', 'f.dci_users_target_id = u.uid');
        $subquery->fields('f', array('entity_id'));
        $query->condition('g.gid', $subquery, 'IN');
        break;
      case 'status':
        switch ($value) {
          case 'not-assigned':
            $subquery = db_select('field_data_dci_users', 'f')->fields('f')->where('f.entity_id = g.gid');
            $query->notExists($subquery);
            break;
          case 'assigned':
            $subquery = db_select('field_data_dci_users', 'f')->fields('f')->where('f.entity_id = g.gid');
            $query->exists($subquery);
            break;
          case 'needs-review':
            $subquery = db_select('field_data_dci_nodes', 'fn');
            $subquery->join('field_data_dci_users', 'fu', 'fu.entity_id = fn.entity_id');
            $subquery->addJoin('LEFT OUTER', 'dci_feedback', 'f', 'fn.dci_nodes_target_id = f.nid AND fu.dci_users_target_id = f.uid');
            $subquery->addField('fn', 'entity_id', 'gid');
            $subquery->addExpression('COUNT(*)', 'total_feedback');
            $subquery->addExpression('COUNT(f.nid)', 'submitted_feedback');
            $subquery->addExpression('COUNT(f.status)', 'reviewed_feedback');
            $subquery->addExpression('COUNT(SIGN(f.status)) - SUM(SIGN(f.status))', 'rejected_feedback');
            $subquery->addExpression('SUM(SIGN(f.status))', 'approved_feedback');
            $subquery->groupBy('fn.entity_id');
            
            $query->join($subquery, 'gf', 'gf.gid = g.gid AND gf.total_feedback <> gf.reviewed_feedback');
            break;
          case 'approved':
            $subquery = db_select('field_data_dci_nodes', 'fn');
            $subquery->join('field_data_dci_users', 'fu', 'fu.entity_id = fn.entity_id');
            $subquery->addJoin('LEFT OUTER', 'dci_feedback', 'f', 'fn.dci_nodes_target_id = f.nid AND fu.dci_users_target_id = f.uid');
            $subquery->addField('fn', 'entity_id', 'gid');
            $subquery->addExpression('COUNT(*)', 'total_feedback');
            $subquery->addExpression('COUNT(f.nid)', 'submitted_feedback');
            $subquery->addExpression('COUNT(f.status)', 'reviewed_feedback');
            $subquery->addExpression('COUNT(SIGN(f.status)) - SUM(SIGN(f.status))', 'rejected_feedback');
            $subquery->addExpression('SUM(SIGN(f.status))', 'approved_feedback');
            $subquery->groupBy('fn.entity_id');
            
            $query->join($subquery, 'gf', 'gf.gid = g.gid AND gf.total_feedback = gf.approved_feedback');
            break;            
        }
        break;
    }
  }
}

/**
 * Form builder: Builds the node administration overview.
 *
 * @see node_admin_nodes_submit()
 * @see node_admin_nodes_validate()
 * @see node_filter_form()
 * @see node_filter_form_submit()
 * @see node_multiple_delete_confirm()
 * @see node_multiple_delete_confirm_submit()
 *
 * @ingroup forms
 */
function dci_admin_groups($params = array()) {

  if (empty($params)) {
    $params = array('mode' => 'admin');
  }
  
  // Build the sortable table header.
  $header = array(
    'title' => array('data' => t('Group title'), 'field' => 'g.title'),
    'item_count' => array('data' => t('Items'), 'field' => 'g.title'),
    'progress' => t('Progress'),
    'assigned_to' => array('data' => t('Assigned to')),
    'due' => array('data' => t('Due'), 'field' => 'g.due'),
    'changed' => array('data' => t('Last updated'), 'field' => 'g.changed', 'sort' => 'desc'),
    'operations' => t('Operations'),
  );

  $query = db_select('dci_group', 'g')->extend('PagerDefault')->extend('TableSort');
  $query->addTag('dci_admin_groups_filter');
  dci_groups_build_filter_query($query);

  $gids = $query
    ->fields('g', array('gid'))
    ->limit(10)
    ->orderByHeader($header)
    ->execute()
    ->fetchCol();
  $groups = dci_group_load_multiple($gids);

  // Prepare the list of groups.
  $destination = drupal_get_destination();
  $options = array();
  $data = array();

  foreach ($groups as $group) {

    $data[$group->gid] = $group;

    $uids = array();
    $nids = array();
    $assigned_to = t('Not Assigned');
    $item_count = isset($group->dci_nodes[LANGUAGE_NONE]) ? count($group->dci_nodes[LANGUAGE_NONE]) : 0;

    if (!empty($group->dci_nodes[LANGUAGE_NONE])) {
      foreach ($group->dci_nodes[LANGUAGE_NONE] as $key => $value) {
        $nids[] = $value['target_id'];
      }
    }

    if (!empty($group->dci_users[LANGUAGE_NONE])) {
      foreach ($group->dci_users[LANGUAGE_NONE] as $key => $value) {
        $uids[] = $value['target_id'];
      }

      $users = db_select('users', 'u')
                ->fields('u', array('uid', 'name'))
                ->condition('uid', $uids, 'IN')
                ->execute()
                ->fetchAllAssoc('uid');
      
      $assigned_to = array('data' => array());
      $w = 0;
      foreach ($users as $uid => $userobj) {
        $assigned_to['data'][] = array(
          '#type' => 'link',
          '#title' => $userobj->name,
          '#href' => 'user/' . $uid,
          '#prefix' => empty($assigned_to['data']) ? NULL : ', ',
          '#weight' => $w++,
        );        
      }
    }

    $total_expected_feedback = count($uids) * count($nids);
    $posted_feedback = 0;
    $progress = 0;

    if ($total_expected_feedback > 0) {

      $fquery = db_select('dci_feedback', 'f')
                ->condition('uid', $uids, 'IN')
                ->condition('nid', $nids, 'IN');
      $fquery->addExpression('COUNT(*)');

      $posted_feedback = $fquery->execute()->fetchField();

      $progress = 100 * ($posted_feedback / $total_expected_feedback);
    }

    $group->progress = $progress;
    $group->completed = $total_expected_feedback > 0 && $posted_feedback == $total_expected_feedback;
    
    $options[$group->gid] = array(
      'title' => array(
        'data' => array(
          '#type' => 'link',
          '#title' => $group->title,
          '#href' => 'admin/dci/groups/' . $group->gid,
        ),
      ),
      'item_count' => array('data' => $item_count),
      'progress' => array('data' => sprintf("%.0f%%", $progress)),
      'assigned_to' => array('data' => $assigned_to),
      'due' => array('data' => format_date($group->due, 'custom', 'm/d/Y')),
      'changed' => array('data' => format_date($group->changed, 'short')),
    );
    
    // Build a list of all the accessible operations for the current node.
    $operations = array();

    if ($params['mode'] === 'add-to-group') {

      $node = $params['node'];

      $operations['add'] = array(
        'title' => t('add'),
        'href' => 'admin/dci/' . $node->nid . '/add-to-group/' . $group->gid,
        'query' => $destination,
      );
    }
    else {
      $operations['edit'] = array(
        'title' => t('edit'),
        'href' => 'admin/dci/groups/' . $group->gid . '/edit',
        'query' => $destination,
      );
      $operations['assign'] = array(
        'title' => t('assign'),
        'href' => 'admin/dci/groups/' . $group->gid . '/assign',
        'query' => $destination,
      );
      $operations['review'] = array(
        'title' => t('review'),
        'href' => 'admin/dci/review',
        'query' => array('group' => $group->gid),
      );
    }
    
    $options[$group->gid]['operations'] = array();
    if (count($operations) > 1) {
      // Render an unordered list of operations links.
      $options[$group->gid]['operations'] = array(
        'data' => array(
          '#theme' => 'links__node_operations',
          '#links' => $operations,
          '#attributes' => array('class' => array('links', 'inline')),
        ),
      );
    }
    elseif (!empty($operations)) {
      // Render the first and only operation as a link.
      $link = reset($operations);
      $options[$group->gid]['operations'] = array(
        'data' => array(
          '#type' => 'link',
          '#title' => $link['title'],
          '#href' => $link['href'],
          '#options' => array('query' => $link['query']),
        ),
      );
    }
  }

  if ($params['mode'] === 'add-to-group') {    
    $form['nodes'] = array(
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $options,
      '#empty' => t('No content available.'),
      '#raw_data' => $data,
    );
  }
  else {
    $form['nodes'] = array(
      '#type' => 'tableselect',
      '#header' => $header,
      '#options' => $options,
      '#empty' => t('No content available.'),
      '#raw_data' => $data,
    );
  }

  $form['pager'] = array('#theme' => 'pager');
  return $form;
}


/**
 * Page callback: Form constructor for the content administration form.
 */
function dci_admin_content_review_form($form, $form_state) {
  
  $form['filter'] = dci_admin_content_review_filter_form();
  $form['#submit'][] = 'dci_admin_content_review_filter_form_submit';
  $form['review'] = dci_admin_content_review();

  return $form;
}

function dci_content_review_filters() {
  // Regular filters
  $filters['title'] = array(
    'title' => t('Title'),
    'type' => 'textfield',
  );

  $filters['user'] = array(
    'title' => t('User'),
    'type' => 'textfield',
  );

  $filters['group'] = array(
    'title' => t('Group'),
    'type' => 'select',
    'options' => array(
      '[any]' => t('any'),
    ),
  );

  $query = db_select('dci_group', 'g')->fields('g', array('gid', 'title'))->orderBy('g.title');

  foreach ($query->execute() as $group) {
    $filters['group']['options'][$group->gid] = $group->title;
  }

  $filters['status'] = array(
    'title' => t('Moderation status'),
    'type' => 'select',
    'options' => array(
      '[any]' => t('any'),
      'needs-review' => t('Needs review'),
      'rejected' => t('Rejected'),
      'approved' => t('Approved'),
    ),
  );
 
  return $filters;
}

/**
 * Returns the node administration filters form array to node_admin_content().
 *
 * @see node_admin_nodes()
 * @see node_admin_nodes_submit()
 * @see node_admin_nodes_validate()
 * @see node_filter_form_submit()
 * @see node_multiple_delete_confirm()
 * @see node_multiple_delete_confirm_submit()
 *
 * @ingroup forms
 */
function dci_admin_content_review_filter_form() {
  if (!isset($_SESSION['dci_content_review_filter'])) {
    $_SESSION['dci_content_review_filter'] = array();
  }
  $session = &$_SESSION['dci_content_review_filter'];
  $filters = dci_content_review_filters();

  $i = 0;
  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Show only items where'),
    '#theme' => 'exposed_filters',
  );

  $form['filters']['status'] = array(
    '#type' => 'container',
    '#attributes' => array('class' => array('container-inline', 'clearfix')),
    '#prefix' => ($i && count($filters) > 0 ? '<div class="additional-filters">' . t('and where') . '</div>' : ''),
  );
  $form['filters']['status']['filters'] = array(
    '#type' => 'container',
    '#attributes' => array('class' => array('container-inline', 'filters')),
  );
  foreach ($filters as $key => $filter) {

    if (isset($_GET[$key])) {
     $session[$key] = $_GET[$key]; 
    }
    
    if ($filter['type'] == 'select') {
      $form['filters']['status']['filters'][$key] = array(
        '#type' => 'select',
        '#options' => $filter['options'],
        '#title' => $filter['title'],
        '#default_value' => isset($session[$key]) ? $session[$key] : '[any]',
      );
    }
    elseif ($filter['type'] == 'textfield') {
      $form['filters']['status']['filters'][$key] = array(
        '#type' => 'textfield',
        '#title' => $filter['title'],
        '#size' => 30,
        '#default_value' => isset($session[$key]) ? $session[$key] : NULL,
      );
    }
  }

  $form['filters']['status']['actions'] = array(
    '#type' => 'actions',
    '#attributes' => array('class' => array('container-inline')),
  );
  if (count($filters)) {
    $form['filters']['status']['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Filter'),
    );
  }
  if (count($session)) {    
    $form['filters']['status']['actions']['reset'] = array('#type' => 'submit', '#value' => t('Reset'));
  }

  drupal_add_js('misc/form.js');

  return $form;
}


/**
 * Form submission handler for node_filter_form().
 *
 * @see node_admin_content()
 * @see node_admin_nodes()
 * @see node_admin_nodes_submit()
 * @see node_admin_nodes_validate()
 * @see node_filter_form()
 * @see node_multiple_delete_confirm()
 * @see node_multiple_delete_confirm_submit()
 */
function dci_admin_content_review_filter_form_submit($form, &$form_state) {  

  switch ($form_state['values']['op']) {
    case t('Filter'):
      $filters = dci_content_review_filters();
    
      // Apply every filter with a valid value.
      foreach ($filters as $filter => $info) {

        if (!empty($form_state['values'][$filter])) {
          switch ($info['type']) {
            
            case 'select':
              if ($form_state['values'][$filter] === '[any]') {
                unset($_SESSION['dci_content_review_filter'][$filter]);
              }
              else {
                // Flatten the options array to accommodate hierarchical/nested options.
                $flat_options = form_options_flatten($info['options']);
                // Only accept valid selections offered on the dropdown, block bad input.
                if (isset($flat_options[$form_state['values'][$filter]])) {
                  $_SESSION['dci_content_review_filter'][$filter] = $form_state['values'][$filter];
                }
              }
              break;

            case 'textfield':
              $_SESSION['dci_content_review_filter'][$filter] = $form_state['values'][$filter];
              break;
          }
        }
        else {
          unset($_SESSION['dci_content_review_filter'][$filter]);
        }
      }
      break;
    case t('Reset'):
      $_SESSION['dci_content_review_filter'] = array();
      $form_state['redirect'] = 'admin/dci/review';
      break;
  }
}

/**
 * Applies filters for node administration filters based on session.
 *
 * @param $query
 *   A SelectQuery to which the filters should be applied.
 */
function dci_content_review_build_filter_query(SelectQueryInterface $query) {
  // Build query
  $filter_data = isset($_SESSION['dci_content_review_filter']) ? $_SESSION['dci_content_review_filter'] : array();
  foreach ($filter_data as $filter => $value) {
    
    switch ($filter) {
      case 'title':
        $query->condition('n.title', $value, 'LIKE');
        break;
      case 'user':
        $query->condition('u.name', $value, 'LIKE');
        break;
      case 'group':
        $subquery = db_select('field_data_dci_nodes', 'fn')->fields('fn', array('dci_nodes_target_id'))->condition('fn.entity_id', $value);
        $query->condition('f.nid', $subquery, 'IN');
        break;
      case 'status':
        switch ($value) {
          case 'needs-review':
            $query->isNull('f.status');
            break;
          case 'rejected':
            $query->condition('f.status', 0);
            break;
          case 'approved':
            $query->condition('f.status', 1);
            break;
        }
        break;
    }
  }
}

/**
 * Form builder: Builds the node administration overview.
 *
 * @see node_admin_nodes_submit()
 * @see node_admin_nodes_validate()
 * @see node_filter_form()
 * @see node_filter_form_submit()
 * @see node_multiple_delete_confirm()
 * @see node_multiple_delete_confirm_submit()
 *
 * @ingroup forms
 */
function dci_admin_content_review() {
  
  // Build the sortable table header.
  $header = array(
    'title' => array('data' => t('Object title'), 'field' => 'n.title'),
    'feedback' => t('Feedback'),
    'user' => array('data' => t('User'), 'field' => 'u.name'),
    'changed' => array('data' => t('Last updated'), 'field' => 'f.changed', 'sort' => 'desc'),
    'status' => array('data' => t('Moderation state'), 'field' => 'f.status'),
    'operations' => t('Operations'),
  );

  $query = db_select('dci_feedback', 'f')->extend('PagerDefault')->extend('TableSort');
  $query->addTag('dci_content');
  dci_content_review_build_filter_query($query);

  $query->join('users', 'u', 'u.uid = f.uid');
  $query->join('node', 'n', 'n.nid = f.nid');

  $query->fields('f', array('fid', 'nid', 'uid', 'status', 'changed'));
  $query->fields('n', array('title'));
  $query->addField('u', 'name', 'username');

  $query->limit(10)->orderByHeader($header);
  $items = $query->execute();

  // Prepare the list of feedback items.
  $destination = drupal_get_destination();
  $options = array();
  foreach ($items as $item) {

    $feedback = dci_feedback_load($item->fid);
    $feedback = entity_view('dci_feedback', array($feedback));
    
    if (isset($feedback['dci_feedback'][$item->fid]['dci_tags'])) {
      $feedback['dci_feedback'][$item->fid]['dci_tags']['#theme'] = 'dci_field_csv';
    }
    $status = 'Needs review';

    if (isset($item->status)) {
      $status = $item->status ? 'Approved' : 'Rejected';
    }

    $options[$item->fid] = array(
      'title' => array(
        'data' => array(
          '#type' => 'link',
          '#title' => $item->title,
          '#href' => 'node/' . $item->nid . '/dci',
        ),
      ),
      'feedback' => array(
        'data' => $feedback,
      ),
      'user' => array(
        'data' => array(
          '#type' => 'link',
          '#title' => $item->username,
          '#href' => 'user/' . $item->uid,
        ),
      ),
      'changed' => format_date($item->changed, 'short'),
      'status' => array(
        'data' => array(
          '#markup' => $status
        ),
      ),
    );
    
    // Build a list of all the accessible operations for the current node.
    $operations = array();

    if (isset($item->status)) {
      $operations['reset'] = array(
        'title' => t('reset'),
        'href' => 'admin/dci/feedback/' . $item->fid . '/reset',
        'query' => $destination,
      );
    }

    if (!isset($item->status) || $item->status == 1) {
      $operations['reject'] = array(
        'title' => t('reject'),
        'href' => 'admin/dci/feedback/' . $item->fid . '/reject',
        'query' => $destination,
      );
    }

    if (!isset($item->status) || $item->status == 0) {
      $operations['approve'] = array(
        'title' => t('approve'),
        'href' => 'admin/dci/feedback/' . $item->fid . '/approve',
        'query' => $destination,
      );
    }
    
    $options[$item->fid]['operations'] = array();
    if (count($operations) > 1) {
      // Render an unordered list of operations links.
      $options[$item->fid]['operations'] = array(
        'data' => array(
          '#theme' => 'links',
          '#links' => $operations,
          '#attributes' => array('class' => array('links', 'inline')),
        ),
      );
    }
    elseif (!empty($operations)) {
      // Render the first and only operation as a link.
      $link = reset($operations);
      $options[$item->fid]['operations'] = array(
        'data' => array(
          '#type' => 'link',
          '#title' => $link['title'],
          '#href' => $link['href'],
          '#options' => array('query' => $link['query']),
        ),
      );
    }
  }

  $form['nodes'] = array(
    '#type' => 'tableselect',
    '#header' => $header,
    '#options' => $options,
    '#empty' => t('No content available.'),
  );

  $form['pager'] = array('#theme' => 'pager');
  return $form;
}

function dci_admin_add_node_to_group_page($node) {

  $result = array(
    'form' => drupal_get_form('dci_admin_add_node_to_group_form', $node),
  );
  return $result;
}

function dci_admin_add_node_to_group_form($form, &$form_state, $node) {

  $form['intro'] = array(
    '#theme' => 'html_tag',
    '#tag' => 'div',
    '#attributes' => array('class' => array('form-item')),
    '#value' => t('Add the content \'%title\' to a group:', array('%title' => $node->title)),
  );
  $form['groups'] = dci_admin_groups(array('mode' => 'add-to-group', 'node' => $node));

  return $form;
}

function dci_admin_add_node_to_group_confirm($form, &$form_state, $node, $group) {

  $form['#node'] = $node;
  $form['#group'] = $group;
  $form['nid'] = array('#type' => 'value', '#value' => $node->nid);
  $form['gid'] = array('#type' => 'value', '#value' => $group->gid);

  return confirm_form($form,
    t('Are you sure you want to add %node_title to %group_title?', array('%node_title' => $node->title, '%group_title' => $group->title)),
    'admin/dci/dashboard',
    '',
    t('Add to group'),
    t('Cancel')
  );
}

/**
 * Executes feedback status change.
 *
 * @see dci_admin_add_node_to_group_confirm()
 */
function dci_admin_add_node_to_group_confirm_submit($form, &$form_state) {
  
  if ($form_state['values']['confirm']) {

    $node = node_load($form_state['values']['nid']);
    $group = dci_group_load($form_state['values']['gid']);

    if ($node && $group) {
      if (empty($group->dci_nodes[LANGUAGE_NONE])) {
        $group->dci_nodes[LANGUAGE_NONE] = array();
      }

      $nids = array_map('reset', $group->dci_nodes[LANGUAGE_NONE]);

      if (!in_array($node->nid, $nids)) {
        $group->dci_nodes[LANGUAGE_NONE][] = array('target_id' => $node->nid);
        dci_group_save($group);
      }
    }
    
    watchdog('dci', 'Content %node_title has been added to DCI group %group_title.', array('%node_title' => $node->title, '%group_title' => $group->title));
    drupal_set_message(t('Content %node_title has been added to DCI group %group_title.', array('%node_title' => $node->title, '%group_title' => $group->title)));
  }

  $form_state['redirect'] = 'admin/dci/dashboard';
}


function dci_admin_assign_users_to_group_page($group) {

  $result = array(
    'form' => drupal_get_form('dci_admin_assign_users_to_group_form', $group),
  );
  return $result;
}

function dci_admin_assign_users_to_group_form($form, &$form_state, $group) {

  $form['intro'] = array(
    '#theme' => 'html_tag',
    '#tag' => 'div',
    '#attributes' => array('class' => array('form-item','pb10')),
    '#value' => t('Assign %title group to the users from the list:', array('%title' => $group->title)),
  );

  $header = array(
    'username' => array('data' => t('Username'), 'field' => 'u.name'),
    'assigned_groups' => array('data' => t('Assigned Groups'), 'field' => 'assigned.assigned_groups'),
    'assigned_items' => array('data' => t('Assigned Items'), 'field' => 'assigned.assigned_items'),
    'member_for' => array('data' => t('Member for'), 'field' => 'u.created', 'sort' => 'desc'),
    'access' => array('data' => t('Last access'), 'field' => 'u.access'),
    'operations' => array('data' => t('Operations')),
  );

  $sq1 = db_select('users_roles', 'ur')->fields('ur', array('uid'));
  $sq1->join('role_permission', 'rp', 'rp.rid = ur.rid');
  $sq1->condition('rp.permission', 'submit content feedback');

  $sq2 = db_select('field_data_dci_users', 'du')
      ->condition('du.deleted', 0)
      ->condition('dn.deleted', 0)
      ->groupBy('du.dci_users_target_id');
  $sq2->join('field_data_dci_nodes', 'dn', 'du.entity_type = dn.entity_type AND du.bundle = dn.bundle AND du.entity_id = dn.entity_id');
  $sq2->addField('du', 'dci_users_target_id', 'uid');
  $sq2->addExpression('COUNT(DISTINCT du.entity_id)', 'assigned_groups');
  $sq2->addExpression('COUNT(DISTINCT dn.dci_nodes_target_id)', 'assigned_items');

  $sq3 = db_select('field_data_dci_users', 'du')
      ->fields('du', array('dci_users_target_id'))
      ->condition('du.deleted', 0)
      ->condition('du.entity_id', $group->gid);

  $query = db_select('users', 'u');
  $query->condition('u.uid', 0, '<>');
  $query->condition('u.status', 1);
  $query->condition('u.uid', $sq1, 'IN');
  $query->condition('u.uid', $sq3, 'NOT IN');

  $count_query = clone $query;
  $count_query->addExpression('COUNT(u.uid)');

  $query->leftJoin($sq2, 'assigned', 'assigned.uid = u.uid');

  $query = $query->extend('PagerDefault')->extend('TableSort');
  $query
    ->fields('u', array('uid', 'name', 'created', 'access'))
    ->fields('assigned', array('assigned_groups', 'assigned_items'))
    ->limit(50)
    ->orderByHeader($header)
    ->setCountQuery($count_query);
  $result = $query->execute();

  $form['bulk'] = array(
    '#type' => 'fieldset',
    '#attributes' => array('class' => array('container-inline')),
  );
  
  $form['bulk']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Assign selected'),
  );

  $form['#group'] = $group;
  $form['gid'] = array('#type' => 'value', '#value' => $group->gid);

  $options = array();

  foreach ($result as $account) {
    $options[$account->uid] = array(
      'username' => theme('username', array('account' => $account)),
      'member_for' => format_interval(REQUEST_TIME - $account->created),
      'assigned_groups' => $account->assigned_groups,
      'assigned_items' => $account->assigned_items,
      'access' =>  $account->access ? t('@time ago', array('@time' => format_interval(REQUEST_TIME - $account->access))) : t('never'),
      'operations' => array('data' => array('#type' => 'link', '#title' => t('assign'), '#href' => "admin/dci/groups/$group->gid/assign/$account->uid", '#options' => array('query' => array('destination' => current_path())))),
    );
  }

  $form['accounts'] = array(
    '#type' => 'tableselect',
    '#header' => $header,
    '#options' => $options,
    '#empty' => t('No people available.'),
  );
  $form['pager'] = array('#markup' => theme('pager'));
  
  return $form;
}


/**
 * Submit the group user bulk assignment form.
 */
function dci_admin_assign_users_to_group_form_submit($form, &$form_state) {

  // Filter out unchecked accounts.
  $accounts = array_filter($form_state['values']['accounts']);

  $group = dci_group_load($form_state['values']['gid']);

  if ($group) {
    if (empty($group->dci_users[LANGUAGE_NONE])) {
      $group->dci_users[LANGUAGE_NONE] = array();
    }

    $uids = array_map('reset', $group->dci_users[LANGUAGE_NONE]);

    $modified = FALSE;

    foreach ($accounts as $uid) {
      if (!in_array($uid, $uids)) {
        $group->dci_users[LANGUAGE_NONE][] = array('target_id' => $uid);
        $modified = TRUE;
      }      
    }

    if ($modified) {
      dci_group_save($group);
    }

    $redirect = "admin/dci/groups/$group->gid/assign";

    if (!empty($_GET['destination'])) {
      $redirect = array($redirect, array('query' => array('destination' => $_GET['destination'])));
      unset($_GET['destination']);
    }
    
    $form_state['redirect'] = $redirect;

    drupal_set_message(t('The %title group has been assigned to @count users.', array('%title' => $group->title, '@count' => count($accounts))));
  }
}

/**
 * Validate the group user bulk assignment form.
 */
function dci_admin_assign_users_to_group_form_validate($form, &$form_state) {
  $form_state['values']['accounts'] = array_filter($form_state['values']['accounts']);
  if (count($form_state['values']['accounts']) == 0) {
    form_set_error('', t('No users selected.'));
  }
}

function dci_admin_assign_user_to_group_confirm($form, &$form_state, $group, $account) {

  $form['#account'] = $account;
  $form['#group'] = $group;
  $form['uid'] = array('#type' => 'value', '#value' => $account->uid);
  $form['gid'] = array('#type' => 'value', '#value' => $group->gid);


  return confirm_form($form,
    t('Are you sure you want to assign group %group_title to user %username?', array('%username' => $account->name, '%group_title' => $group->title)),
    'admin/dci/groups',
    '',
    t('Add to group'),
    t('Cancel')
  );
}

/**
 * Executes feedback status change.
 *
 * @see dci_admin_add_node_to_group_confirm()
 */
function dci_admin_assign_user_to_group_confirm_submit($form, &$form_state) {
  
  if ($form_state['values']['confirm']) {

    $account = user_load($form_state['values']['uid']);
    $group = dci_group_load($form_state['values']['gid']);

    if ($account && $group) {
      if (empty($group->dci_users[LANGUAGE_NONE])) {
        $group->dci_users[LANGUAGE_NONE] = array();
      }

      $uids = array_map('reset', $group->dci_users[LANGUAGE_NONE]);

      if (!in_array($account->uid, $uids)) {
        $group->dci_users[LANGUAGE_NONE][] = array('target_id' => $account->uid);
        dci_group_save($group);
      }
    }
    
    watchdog('dci', 'DCI group %group_title has been assigned to user %username.', array('%username' => $account->name, '%group_title' => $group->title));
    drupal_set_message(t('DCI group %group_title has been assigned to user %username.', array('%username' => $account->name, '%group_title' => $group->title)));
  }

  $form_state['redirect'] = 'admin/dci/groups';
}